@@ -8,7 +8,7 @@
 # License   : Revised BSD License
 # -----------------------------------------------------------------------------
 # Creation  : 26-Apr-2010
-# Last mod  : 30-Jul-2012
+# Last mod  : 11-Sep-2012
 # -----------------------------------------------------------------------------
 
 """
@@ -42,7 +42,7 @@ from __future__ import with_statement
 import base64, bz2, hashlib, os, random, sys, re, string, tempfile, subprocess, types, functools, StringIO
 import fabric, fabric.api, fabric.operations, fabric.context_managers
 
-VERSION         = "0.3.2"
+VERSION         = "0.4.0"
 RE_SPACES       = re.compile("[\s\t]+")
 MAC_EOL         = "\n"
 UNIX_EOL        = "\n"
@@ -51,11 +51,14 @@ MODE_LOCAL      = "CUISINE_MODE_LOCAL"
 MODE_SUDO       = "CUISINE_MODE_SUDO"
 SUDO_PASSWORD   = "CUISINE_SUDO_PASSWORD"
 OPTION_PACKAGE  = "CUISINE_OPTION_PACKAGE"
+OPTION_PYTHON_PACKAGE  = "CUISINE_OPTION_PYTHON_PACKAGE"
 AVAILABLE_OPTIONS = dict(
-	package=["apt", "yum"]
+	package=["apt", "yum"],
+    python_package=["easy_install","pip"]
 )
 DEFAULT_OPTIONS = dict(
-	package="apt"
+	package="apt",
+    python_package="pip"
 )
 
 
@@ -133,6 +136,12 @@ def select_package( option=None ):
 		fabric.api.env[OPTION_PACKAGE] = option
 	return (fabric.api.env[OPTION_PACKAGE], supported)
 
+def select_python_package( option=None ):
+    supported = AVAILABLE_OPTIONS["python_package"]
+    if not (option is None):
+        assert option in supported, "Option must be one of: %s"  % (supported)
+        fabric.api.env[OPTION_PYTHON_PACKAGE] = option
+    return (fabric.api.env[OPTION_PYTHON_PACKAGE], supported)
 # =============================================================================
 #
 # RUN/SUDO METHODS
@@ -142,7 +151,7 @@ def select_package( option=None ):
 def run_local(command, sudo=False, shell=True, pty=True, combine_stderr=None):
 	"""
 	Local implementation of fabric.api.run() using subprocess.
-	
+
 	Note: pty option exists for function signature compatibility and is
 	      ignored.
 	"""
@@ -221,9 +230,9 @@ def dispatch(prefix=None):
 	def dispatch_wrapper(function, prefix=prefix):
 		def wrapper(*args, **kwargs):
 			function_name = function.__name__
-			_prefix       = prefix or function_name.split("_")[0].upper().replace(".","_")
-			select        = fabric.api.env.get("CUISINE_OPTION_" + _prefix)
-			assert select, "No option defined for: %s, call select_%s(<YOUR OPTION>) to set it" % (_prefix, prefix.lower().replace(".","_"))
+			_prefix       = prefix or function_name.split("_")[0].replace(".","_")
+			select        = fabric.api.env.get("CUISINE_OPTION_" + _prefix.upper())
+			assert select, "No option defined for: %s, call select_%s(<YOUR OPTION>) to set it" % (_prefix.upper(), prefix.lower().replace(".","_"))
 			function_name = function.__name__ + "_" + select
 			specific      = eval(function_name)
 			if specific:
@@ -392,14 +401,23 @@ def file_write(location, content, mode=None, owner=None, group=None, sudo=None,
 		if is_local():
 			run('cp "%s" "%s"'%(local_path,location))
 		else:
-			fabric.operations.put(local_path, location, use_sudo=use_sudo)
+			# FIXME: Put is not working properly, I often get stuff like:
+			# Fatal error: sudo() encountered an error (return code 1) while executing 'mv "3dcf7213c3032c812769e7f355e657b2df06b687" "/etc/authbind/byport/80"'
+			#fabric.operations.put(local_path, location, use_sudo=use_sudo)
+			# Hides the output, which is especially important
+			with fabric.context_managers.settings(
+				fabric.api.hide('warnings', 'running', 'stdout'),
+				warn_only=True
+			):
+				# We send the data as BZipped Base64
+				run("echo '%s' | base64 -d | bzcat > \"%s\"" % (base64.b64encode(bz2.compress(content)), location))
 	# Remove the local temp file
 	os.close(fd)
 	os.unlink(local_path)
 	# Ensures that the signature matches
 	if check:
 		file_sig = file_sha256(location)
-		assert sig == file_sig, "File content does not matches file: %s, got %s, expects %s" % (location, sig, file_sig)
+		assert sig == file_sig, "File content does not matches file: %s, got %s, expects %s" % (location, repr(file_sig), repr(sig))
 	file_attribs(location, mode=mode, owner=owner, group=group)
 
 def file_ensure(location, mode=None, owner=None, group=None, recursive=False):
@@ -471,7 +489,8 @@ def file_sha256(location):
 	# NOTE: In some cases, sudo can output errors in here -- but the errors will
 	# appear before the result, so we simply split and get the last line to
 	# be on the safe side.
-	return run('sha256sum "%s" | cut -d" " -f1' % (location)).replace("\n","").strip()
+	sig = run('sha256sum "%s" | cut -d" " -f1' % (location)).split("\n")
+	return sig[-1].strip()
 
 # =============================================================================
 #
@@ -580,25 +599,25 @@ def repository_ensure_yum(repository):
 	raise Exception("Not implemented for Yum")
 
 def package_upgrade_yum():
-	sudo("yum --assumeyes update")
+	sudo("yum -y update")
 
 def package_update_yum(package=None):
 	if package == None:
-		sudo("yum --assumeyes update")
+		sudo("yum -y update")
 	else:
 		if type(package) in (list, tuple):
 			package = " ".join(package)
-		sudo("yum --assumeyes upgrade " + package)
+		sudo("yum -y upgrade " + package)
 
 def package_upgrade_yum(package=None):
-	sudo("yum --assumeyes upgrade")
+	sudo("yum -y upgrade")
 
 def package_install_yum(package, update=False):
 	if update:
-		sudo("yum --assumeyes update")
+		sudo("yum -y update")
 	if type(package) in (list, tuple):
 		package = " ".join(package)
-	sudo("yum --assumeyes install %s" % (package))
+	sudo("yum -y install %s" % (package))
 
 def package_ensure_yum(package, update=False):
 	status = run("yum list installed %s ; true" % package)
@@ -610,7 +629,132 @@ def package_ensure_yum(package, update=False):
 		return True
 
 def package_clean_yum(package=None):
-	sudo("yum --assumeyes clean all")
+	sudo("yum -y clean all")
+
+
+# =============================================================================
+#
+# PYTHON PACKAGE OPERATIONS
+#
+# =============================================================================
+
+@dispatch('python_package')
+def python_package_upgrade(package):
+	'''
+    Upgrades the defined python package.
+    '''
+
+@dispatch('python_package')
+def python_package_install(package=None):
+	'''
+    Installs the given python package/list of python packages.
+    '''
+
+@dispatch('python_package')
+def python_package_ensure(package):
+	'''
+    Tests if the given python package is installed, and installes it in
+	case it's not already there.
+    '''
+
+@dispatch('python_package')
+def python_package_remove(package):
+	'''
+    Removes the given python package.
+    '''
+
+# -----------------------------------------------------------------------------
+# PIP PYTHON PACKAGE MANAGER
+# -----------------------------------------------------------------------------
+
+def python_package_upgrade_pip(package,E=None):
+    '''
+    The "package" argument, defines the name of the package that will be upgraded.
+    The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
+    path to a virtualenv. If provided, it will be added to the pip call.
+    '''
+    if E:
+        E='-E %s' %E
+    else:
+        E=''   
+    run('pip upgrade %s %s' %(E,package))
+
+def python_package_install_pip(package=None,r=None,pip=None):
+    '''
+    The "package" argument, defines the name of the package that will be installed.
+    The argument "r" referes to the requirements file that will be used by pip and
+    is equivalent to the "-r" parameter of pip.
+    Either "package" or "r" needs to be provided
+    The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
+    path to a virtualenv. If provided, it will be added to the pip call.
+    '''
+    pip=pip or fabric.api.env.env.get('pip','pip')
+    if package:
+        run('%s install %s' %(pip,package))
+    elif r:
+        run('%s install -r %s' %(pip,r))
+    else:
+        raise Exception("Either a package name or the requirements file has to be provided.")
+
+def python_package_ensure_pip(package=None,r=None, pip=None):
+    '''
+    The "package" argument, defines the name of the package that will be ensured.
+    The argument "r" referes to the requirements file that will be used by pip and
+    is equivalent to the "-r" parameter of pip.
+    Either "package" or "r" needs to be provided
+    The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
+    path to a virtualenv. If provided, it will be added to the pip call.
+    '''
+    #FIXME: At the moment, I do not know how to check for the existence of a pip package and
+    # I am not sure if this really makes sense, based on the pip built in functionality. 
+    # So I just call the install functions
+    pip=pip or fabric.api.env.env.get('pip','pip')
+    python_package_install_pip(package,r,pip)
+
+def python_package_remove_pip(package, E=None):
+    '''
+    The "package" argument, defines the name of the package that will be ensured.
+    The argument "r" referes to the requirements file that will be used by pip and
+    is equivalent to the "-r" parameter of pip.
+    Either "package" or "r" needs to be provided
+    The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
+    path to a virtualenv. If provided, it will be added to the pip call. 
+    '''
+    pip=pip or fabric.api.env.env.get('pip','pip')
+    return run('%s uninstall %s' %(pip,package))
+
+
+# -----------------------------------------------------------------------------
+# EASY_INSTALL PYTHON PACKAGE MANAGER
+# -----------------------------------------------------------------------------
+
+def python_package_upgrade_easy_install(package):
+    '''
+    The "package" argument, defines the name of the package that will be upgraded.
+    '''
+    run('easy_install --upgrade %s' %package)
+
+def python_package_install_easy_install(package):
+    '''
+    The "package" argument, defines the name of the package that will be installed.
+    '''
+    sudo('easy_install %s' %package)
+
+def python_package_ensure_easy_install(package):
+    '''
+    The "package" argument, defines the name of the package that will be ensured.
+    '''
+    #FIXME: At the moment, I do not know how to check for the existence of a py package and
+    # I am not sure if this really makes sense, based on the easy_install built in functionality. 
+    # So I just call the install functions
+    python_package_install_easy_install(package)
+
+def python_package_remove_easy_install(package):
+    '''
+    The "package" argument, defines the name of the package that will be removed.
+    '''
+    #FIXME: this will not remove egg file etc.
+    run('easy_install -m %s' %package) 
 
 # =============================================================================
 #
@@ -643,7 +787,7 @@ def user_passwd(name, passwd, encrypted_passwd=False):
 	"""Sets the given user password."""
 	encoded_password = base64.b64encode("%s:%s" % (name, passwd))
 	encryption = " -e" if encrypted_passwd else ""
-	sudo("echo %s | base64 --decode | chpasswd%s" % encoded_password, encryption)
+	sudo("echo %s | base64 --decode | chpasswd%s" % (encoded_password, encryption))
 
 def user_create(name, passwd=None, home=None, uid=None, gid=None, shell=None,
 				uid_min=None, uid_max=None, encrypted_passwd=False):
@@ -669,20 +813,25 @@ def user_create(name, passwd=None, home=None, uid=None, gid=None, shell=None,
 	if passwd:
 		user_passwd(name,passwd,encrypted_passwd)
 
-def user_check(name):
+def user_check(name=None, uid=None):
 	"""Checks if there is a user defined with the given name,
 	returning its information as a
 	'{"name":<str>,"uid":<str>,"gid":<str>,"home":<str>,"shell":<str>}'
 	or 'None' if the user does not exists."""
-	d = sudo("cat /etc/passwd | egrep '^%s:' ; true" % (name))
-	s = sudo("cat /etc/shadow | egrep '^%s:' | awk -F':' '{print $2}'" % (name))
+	assert name!=None or uid!=None,     "user_check: either `uid` or `name` should be given"
+	assert name is None or uid is None,"user_check: `uid` and `name` both given, only one should be provided"
+	if   name != None:
+		d = sudo("cat /etc/passwd | egrep '^%s:' ; true" % (name))
+	elif uid != None:
+		d = sudo("cat /etc/passwd | egrep '^.*:.*:%s:' ; true" % (uid))
 	results = {}
+	s = None
 	if d:
 		d = d.split(":")
 		assert len(d) >= 7, "/etc/passwd entry is expected to have at least 7 fields, got %s in: %s" % (len(d), ":".join(d))
 		results = dict(name=d[0], uid=d[2], gid=d[3], home=d[5], shell=d[6])
-	if s:
-		results['passwd'] = s
+		s = sudo("cat /etc/shadow | egrep '^%s:' | awk -F':' '{print $2}'" % (results['name']))
+		if s: results['passwd'] = s
 	if results:
 		return results
 	else:
